using System;
using System.Collections.Generic;
using System.Text;
using SWConfigDataClientLib;
using SWConfigDataClientLib.Data;
using SWConfigDataClientLib.Proxies.Users;
using SWConfigDataSharedLib.Exceptions;

namespace EditUserAttributes
{
    class Program
    {
        private LibManager m_LibManager;

        //------------------------------------------------------------------------------
        
        static void Main(string[] args)
        {
            LibManager libManager = new LibManager();
            libManager.BaseProtocol = SProxyObject.BaseProtocol.TCP;

            // This sample program connects to the local SwyxServer via Windows Authentication
            libManager.AuthenticationMode = SProxyObject.AuthenticationMode.Trusted;
            libManager.WSBaseUrl = "localhost";

            // If you want to connect to a remote SwyxServer via
            // UsernamePassword Authentication use the following code:
            /*
            libManager.AuthenticationMode = SProxyObject.AuthenticationMode.UsernamePassword;
            libManager.WSBaseUrl = "<YourRemoteSwyxServerAddress>";
            libManager.Username = "ippbx:<NameOfAnSwyxWareUser>";
            libManager.Password = "<PasswordOftheSwyxWareUser>";
            */

            Program program = new Program(libManager);

            if (program.TestConnection())
            {
                program.EditUserCommentsWithOneUpdate();
                program.EditUserCommentsWithSingleUpdates();
            }

            Console.ReadLine();
        }

        //------------------------------------------------------------------------------

        public Program(LibManager libManager)
        {
            m_LibManager = libManager;
        }

        //------------------------------------------------------------------------------

        public bool TestConnection()
        {
            try
            {
                SWConfigDataClientLib.Proxies.IppbxServer.IppbxServerFacade facade;
                facade = m_LibManager.GetIppbxServerFacade();
                
                int DBScheme = facade.GetDBSchema();
                
                facade.FreeForReuse();
                
                Console.WriteLine(string.Format("Conection established, DBSchema:{0}",DBScheme));

                return true;                
            }
            catch (SBaseException ex)
            {
                Console.WriteLine(string.Format("Conection failed with CDS Exception: {0}", ex.MessageSummary));
            }
            catch (Exception ex)
            {
                Console.WriteLine(string.Format("Conection failed: {0}", ex.Message));
            }
            return false;
        }

        //------------------------------------------------------------------------------

        public void EditUserCommentsWithOneUpdate()
        {
            try
            {
                UserEnum userEnum = m_LibManager.GetUserEnum();

                userEnum.ExecuteNameFilter("%", new OrderByList());
                
                foreach(UserEntry userEntry in userEnum.PrimaryCollection)
                {
                    Console.WriteLine(string.Format("User: {0}", userEntry.Name));

                    if (userEntry.IsCommentNull)
                    {
                        userEntry.Comment = "has been read";
                    }
                }

                userEnum.Update();
                userEnum.FreeForReuse();
            }
            catch (SBaseException ex)
            {
                Console.WriteLine(string.Format("Conection failed with CDS Exception: {0}", ex.MessageSummary));
            }
            catch (Exception ex)
            {
                Console.WriteLine(string.Format("Conection failed: {0}", ex.Message));
            }
        }

        //------------------------------------------------------------------------------

        public void EditUserCommentsWithSingleUpdates()
        {
            try
            {
                UserEnum userEnum = m_LibManager.GetUserEnum();

                userEnum.ExecuteNameFilter("%", new OrderByList());
                
                foreach(UserEntry userEntry in userEnum.PrimaryCollection)
                {
                    Console.WriteLine(string.Format("User: {0}", userEntry.Name));

                    if (!userEntry.IsCommentNull)
                    {
                        if (userEntry.Comment.Equals("has been read"))
                        {
                            userEntry.Comment = "has been read twice";
                            try
                            {
                                userEntry.Update(true);
                            }
                            catch (SBaseException ex)
                            {
                                Console.WriteLine(string.Format("Update of user {0} failed with exception {1}.", userEntry.Name, ex.MessageSummary));
                            }
                        }
                    }
                }

                userEnum.FreeForReuse();
            }
            catch (SBaseException ex)
            {
                Console.WriteLine(string.Format("Conection failed with CDS Exception: {0}", ex.MessageSummary));
            }
            catch (Exception ex)
            {
                Console.WriteLine(string.Format("Conection failed: {0}", ex.Message));
            }
        }

        //------------------------------------------------------------------------------
    }
}
