﻿using System;
using System.IO;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using SWConfigDataClientLib;
using SWConfigDataClientLib.Proxies.Admin;
using SWConfigDataClientLib.Proxies.Files;
using SWConfigDataClientLib.Proxies.PhoneClient;
using SWConfigDataSharedLib.Exceptions;

namespace FileDemo
{
    public partial class frmMain : Form
    {
        #region CDS

        private LibManager m_LibManager;
        private AdminFacade m_AdminFacade;
        private PhoneClientFacade m_PhoneClientFacade;
        private FilesFacade m_FilesFacade;
        private FileListEntrySortableCollection m_FileList;
        private bool m_Connected;

        #endregion

        //------------------------------------------------------------------------------------

        public frmMain()
        {
            InitializeComponent();
        }

        ~frmMain()
        {
            if (m_FilesFacade != null)
            {
                m_FilesFacade.FreeForReuse();
                m_FilesFacade = null;
            }

            if (m_PhoneClientFacade != null)
            {
                m_PhoneClientFacade.FreeForReuse();
                m_PhoneClientFacade = null;
            }

            if (m_AdminFacade != null)
            {
                m_AdminFacade.FreeForReuse();
                m_AdminFacade = null;
            }

            if (m_LibManager != null)
            {
                m_LibManager.FreeForReuse();
                m_LibManager = null;
            }
        }

        //------------------------------------------------------------------------------------
        // CONTROLS
        //------------------------------------------------------------------------------------

        private void cmdLogin_Click(object sender, EventArgs e)
        {
            if (ConnectToCDS())
            {
                txtName.Enabled = false;
                txtPassword.Enabled = false;
                txtServer.Enabled = false;
                cmdLogin.Enabled = false;                
                cmdLoadSkins.Enabled = true;                
                cbxSkinList.Enabled = true;
            }
        }

        //------------------------------------------------------------------------------------

       private void cmdLoadSkins_Click(object sender, EventArgs e)
        {
            if (LoadSkins())
            {
                cmdLoadSkins.Enabled = false;
                cmdDownloadSkin.Enabled = true;
                cmdUploadSkin.Enabled = true;
            }
        }

        //------------------------------------------------------------------------------------

        private void cmdDownloadSkin_Click(object sender, EventArgs e)
        {
            SaveFileDialog dialog = new SaveFileDialog();
            dialog.FileName = cbxSkinList.Text;
            dialog.Filter = "Skin files|*.cab|All files|*.*";
            DialogResult res = dialog.ShowDialog();
            if (res == DialogResult.OK)
            {
                DownloadSkin(cbxSkinList.SelectedIndex, dialog.FileName);
            }
        }

        //------------------------------------------------------------------------------------

        private void cmdUploadSkin_Click(object sender, EventArgs e)
        {
            try
            {
                if (!m_PhoneClientFacade.HasRole(SWConfigDataClientLib.WSPhoneClientFacade.BuildInRoles.FileRW))
                {
                    throw new SSecurityException("The user has insufficient privileges");
                }

                OpenFileDialog dialog = new OpenFileDialog();
                dialog.Filter = "Skin files|*.cab|All files|*.*";
                DialogResult res = dialog.ShowDialog();
                if (res == DialogResult.OK)
                {
                    UploadSkin(dialog.FileName, dialog.SafeFileName);
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Information", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        //------------------------------------------------------------------------------------
        // CDS Methods
        //------------------------------------------------------------------------------------

        public bool ConnectToCDS()
        {
            if (m_Connected)
                return true;

            m_Connected = false;

            try
            {
                if (m_LibManager == null)
                    m_LibManager = new LibManager();

                if (String.IsNullOrEmpty(txtName.Text) || String.IsNullOrEmpty(txtPassword.Text) || String.IsNullOrEmpty(txtServer.Text))
                {
                    return false;
                }

                m_LibManager.Username = "ippbx:" + txtName.Text;
                m_LibManager.Password = txtPassword.Text;
                m_LibManager.WSBaseUrl = txtServer.Text;
                m_LibManager.AuthenticationMode = SProxyObject.AuthenticationMode.UsernamePassword;
                m_LibManager.BaseProtocol = SProxyObject.BaseProtocol.TCP;

                m_AdminFacade = m_LibManager.GetAdminFacade();                
                string username = m_AdminFacade.GetCurrentUserName();

                m_PhoneClientFacade = m_LibManager.GetPhoneClientFacade();
                m_FilesFacade = m_LibManager.GetFilesFacade();

                m_Connected = true;
                return true;
            }
            catch (SWConfigDataSharedLib.Exceptions.UserDbException ex)
            {
                MessageBox.Show(ex.Message, "Information", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (SWConfigDataSharedLib.Exceptions.SBaseException ex)
            {
                MessageBox.Show(ex.Message, "Information", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Information", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

            return false;
        }

        //------------------------------------------------------------------------------------

        public bool LoadSkins()
        {
            try
            {
                cbxSkinList.Items.Clear();
                m_FileList = null;
                
                // Get skins from global scope
                m_FileList = m_FilesFacade.GetFileList("%", FileScope.Global, PreDefinedFileCategory.Skins, 0, true).CreateSortableFilterCollection();

                // Add skins from SystemDefault scope to the same list
                foreach (FileListEntry entry in m_FilesFacade.GetFileList("%", FileScope.SystemDefault, PreDefinedFileCategory.Skins, 0, true).CreateSortableFilterCollection())
                {   
                    m_FileList.Add(entry);
                }

                if (m_FileList.Count == 0)
                {
                    throw new SBaseException("No skins found");
                }

                // Add skins to combobox
                foreach (FileListEntry entry in m_FileList)
                {
                    cbxSkinList.Items.Add(entry.Name);
                }

                // Select first skin in the combobox
                if (cbxSkinList.Items.Count > 0)
                {
                    cbxSkinList.SelectedIndex = 0;
                }

                return true;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Information", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            return false;
        }

        //------------------------------------------------------------------------------------

        public void DownloadSkin(int skinIndexId, string savePath)
        {
            try
            {
                string localTempFilePath = m_FilesFacade.GetIpPbxFile(m_FileList[skinIndexId]);
                File.Copy(localTempFilePath, savePath);
                File.SetAttributes(savePath, FileAttributes.Normal);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Information", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        //------------------------------------------------------------------------------------

        public void UploadSkin(string sourcePath, string fileName)
        {
            try
            {
                m_FilesFacade.AddIpPbxFile(fileName,
                    FileScope.Global,
                    PreDefinedFileCategory.Skins,
                    0,
                    false,
                    false,
                    "Uploaded with CDS Client SDK FileDemo Sample",
                    false,
                    false,
                    "en-GB",
                    sourcePath,
                    true);

                LoadSkins();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Information", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        //------------------------------------------------------------------------------------

    }
}
