﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using SWConfigDataClientLib;
using SWConfigDataClientLib.Proxies.Admin;
using SWConfigDataClientLib.WSUserPhoneBook;

namespace UserAdminView
{
    public partial class frmMain : Form
    {
        #region CDS

        private LibManager m_LibManager;
        private AdminFacade m_AdminFacade;
        private bool m_Connected;

        #endregion

        #region Data Members
        UserDeviceSmallMap m_DeviceMap;
        UserAdminView1EntrySortableCollection m_Users;
        #endregion

        //------------------------------------------------------------------------------------

        public frmMain()
        {
            InitializeComponent();
            m_Connected = false;
        }

        ~frmMain()
        {
            if (m_AdminFacade != null)
            {
                m_AdminFacade.FreeForReuse();
                m_AdminFacade = null;
            }

            if (m_LibManager != null)
            {
                m_LibManager.FreeForReuse();
                m_LibManager = null;
            }
        }

        //------------------------------------------------------------------------------------

        public bool ConnectToCDS()
        {
            if (m_Connected)
                return true;

            m_Connected = false;

            try
            {
                if (m_LibManager == null)
                    m_LibManager = new LibManager();

                if (String.IsNullOrEmpty(txtName.Text) || String.IsNullOrEmpty(txtPassword.Text) || String.IsNullOrEmpty(txtServer.Text))
                {
                    return false;
                }

                m_LibManager.Username = "ippbx:" + txtName.Text;
                m_LibManager.Password = txtPassword.Text;
                m_LibManager.WSBaseUrl = txtServer.Text;
                m_LibManager.AuthenticationMode = SProxyObject.AuthenticationMode.UsernamePassword;
                m_LibManager.BaseProtocol = SProxyObject.BaseProtocol.TCP;

                m_AdminFacade = m_LibManager.GetAdminFacade();
                string username = m_AdminFacade.GetCurrentUserName();

                m_Connected = true;
                return true;
            }
            catch (SWConfigDataSharedLib.Exceptions.UserDbException ex)
            {
                MessageBox.Show(ex.Message, "Information", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (SWConfigDataSharedLib.Exceptions.SBaseException ex)
            {
                MessageBox.Show(ex.Message, "Information", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Information", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

            return false;
        }

        //------------------------------------------------------------------------------------

        private void cmdLogin_Click(object sender, EventArgs e)
        {
            if (ConnectToCDS())
            {
                txtName.Enabled = false;
                txtPassword.Enabled = false;
                txtServer.Enabled = false;
                cmdLogin.Enabled = false;                
                cmdUpdate.Enabled = false;
                cmdLoad.Enabled = true;
            }
        }

        //------------------------------------------------------------------------------------

        private void LoadView()
        {
            int i_RowId = 0;

            try
            {
                dgvPhonebook.Rows.Clear();

                m_Users = m_AdminFacade.GetUserAdminView1(false, out m_DeviceMap).CreateSortableFilterCollection();

                if (m_Users.Count < 1)
                {
                    tslCounter.Text = "Results: 0";
                    return;
                }

                tslCounter.Text = "Results: " + m_Users.Count.ToString();

                foreach (UserAdminView1Entry entry in m_Users)
                {
                    i_RowId = dgvPhonebook.Rows.Add();
                    dgvPhonebook.Rows[i_RowId].Cells[0].Value = entry.UserID;
                    dgvPhonebook.Rows[i_RowId].Cells[1].Value = entry.Name;
                    dgvPhonebook.Rows[i_RowId].Cells[2].Value = entry.Comment;
                    dgvPhonebook.Rows[i_RowId].Cells[3].Value = entry.EMailAddr;
                    dgvPhonebook.Rows[i_RowId].Cells[4].Value = entry.Locked;
                    dgvPhonebook.Rows[i_RowId].Cells[5].Value = entry.LoginTimeLocal;
                }

                cmdLoad.Enabled = false;
                cmdUpdate.Enabled = true;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString(), "Information", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }            
        }

        //------------------------------------------------------------------------------------

        private void cmdLoad_Click(object sender, EventArgs e)
        {
            LoadView();
        }
        
        //------------------------------------------------------------------------------------

        private void cmdUpdate_Click(object sender, EventArgs e)
        {
            LoadView();
        }

        //------------------------------------------------------------------------------------

    }
}
