﻿using System;
using System.Windows.Forms;
using SWConfigDataClientLib;
using SWConfigDataClientLib.Proxies.Admin;
using SWConfigDataClientLib.Proxies.PhoneClient;
using SWConfigDataSharedLib.Exceptions;

namespace ViewActiveCalls
{
    public partial class frmMain : Form
    {        
        private LibManager m_LibManager;        
        private AdminFacade m_AdminFacade;
        private ActiveCallEntrySortableCollection m_ActiveCalls;
        private PhoneClientFacade m_PhoneClientFacade;
        private bool m_Connected;
        
        //------------------------------------------------------------------------------------

        public frmMain()
        {
            InitializeComponent();
            m_Connected = false;
            m_AdminFacade = null;
            m_PhoneClientFacade = null;
        }

        ~frmMain()
        {
            if (m_PhoneClientFacade != null)
            {
                m_PhoneClientFacade.FreeForReuse();
                m_PhoneClientFacade = null;
            }

            if (m_AdminFacade != null)
            {
                m_AdminFacade.FreeForReuse();
                m_AdminFacade = null;
            }

            if (m_LibManager != null)
            {
                m_LibManager.FreeForReuse();
                m_LibManager = null;
            }
        }

        //------------------------------------------------------------------------------------

        private void cmdLogin_Click(object sender, EventArgs e)
        {
            if (ConnectToCDS())
            {
                txtName.Enabled = false;
                txtPassword.Enabled = false;
                txtServer.Enabled = false;
                cmdLogin.Enabled = false;
                cmdLoad.Enabled = true;
            }
        }

        //------------------------------------------------------------------------------------

        private void cmdLoad_Click(object sender, EventArgs e)
        {
            if (m_AdminFacade == null)
                m_AdminFacade = m_LibManager.GetAdminFacade();
            m_ActiveCalls = m_AdminFacade.GetActiveCallsView().CreateSortableFilterCollection();
            ShowValues();
        }

        //------------------------------------------------------------------------------------

        private void ShowValues()
        {
            int i_RowId = 0;

            tslCounter.Text = "Active Calls: " + m_ActiveCalls.Count.ToString();
            dgvPhonebook.Rows.Clear();

            foreach (ActiveCallEntry callEntry in m_ActiveCalls)
            {                
                i_RowId = dgvPhonebook.Rows.Add();
                dgvPhonebook.Rows[i_RowId].Cells[0].Value = callEntry.CallID.ToString();
                dgvPhonebook.Rows[i_RowId].Cells[1].Value = callEntry.CallStatus.ToString();
                dgvPhonebook.Rows[i_RowId].Cells[2].Value = callEntry.Start.ToString();
                dgvPhonebook.Rows[i_RowId].Cells[3].Value = callEntry.Duration.ToString();
                dgvPhonebook.Rows[i_RowId].Cells[4].Value = callEntry.CalledName;
                dgvPhonebook.Rows[i_RowId].Cells[5].Value = callEntry.CalledNumber;
                dgvPhonebook.Rows[i_RowId].Cells[6].Value = callEntry.DestinationDeviceName;
                dgvPhonebook.Rows[i_RowId].Cells[7].Value = callEntry.DestinationName;
                dgvPhonebook.Rows[i_RowId].Cells[8].Value = callEntry.DestinationNumber;                
                dgvPhonebook.Rows[i_RowId].Cells[9].Value = callEntry.OriginationDeviceName;
                dgvPhonebook.Rows[i_RowId].Cells[10].Value = callEntry.OriginationName;
                dgvPhonebook.Rows[i_RowId].Cells[11].Value = callEntry.OriginationNumber;
                dgvPhonebook.Rows[i_RowId].Cells[12].Value = callEntry.ProjectNumber;
                dgvPhonebook.Rows[i_RowId].Cells[13].Value = callEntry.Charges;
            }

            m_ActiveCalls = null;
        }

        //------------------------------------------------------------------------------------

        public bool ConnectToCDS()
        {
            if (m_Connected)
                return true;

            try
            {
                if (m_LibManager == null)
                    m_LibManager = new LibManager();

                if (String.IsNullOrEmpty(txtName.Text) || String.IsNullOrEmpty(txtPassword.Text) || String.IsNullOrEmpty(txtServer.Text))
                {
                    throw new ArgumentException("Missing or invalid login parameters");
                }

                m_LibManager.Username = "ippbx:" + txtName.Text;
                m_LibManager.Password = txtPassword.Text;
                m_LibManager.WSBaseUrl = txtServer.Text;
                m_LibManager.AuthenticationMode = SProxyObject.AuthenticationMode.UsernamePassword;
                m_LibManager.BaseProtocol = SProxyObject.BaseProtocol.TCP;

                m_PhoneClientFacade = m_LibManager.GetPhoneClientFacade();
                string username = m_PhoneClientFacade.GetCurrentUserName();

                // CHECKING USER RIGHTS
                // User must have the role "ActiveCallsAdminRO" to view the list of active calls
                // If the user has the additional role "ActiveCallsAdminRW" he can also terminate active calls

                if (m_PhoneClientFacade.HasRole(SWConfigDataClientLib.WSPhoneClientFacade.BuildInRoles.ActiveCallsAdminRO))
                {
                    m_Connected = true;
                    return true;
                }
                else throw new SSecurityException("Insufficient privileges");
            }
            catch (UserDbException ex)
            {
                MessageBox.Show(ex.Message, "Information", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (SSecurityException ex)
            {
                MessageBox.Show(ex.Message, "Information", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (SBaseException ex)
            {
                MessageBox.Show(ex.Message, "Information", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Information", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }            
            return false;
        }

        //------------------------------------------------------------------------------------

    }
}
